@@ -67,6 +67,7 @@ struct a2dp_data {
 
 struct headset_data {
 	headset_lock_t lock;
+	gboolean locked;
 };
 
 struct unix_client {
@@ -94,6 +95,9 @@ static void client_free(struct unix_client *client)
 {
 	struct a2dp_data *a2dp;
 
+	if (client->req_id)
+		client->cancel(client->dev, client->req_id);
+
 	switch (client->type) {
 	case TYPE_SINK:
 	case TYPE_SOURCE:
@@ -250,6 +254,8 @@ static void headset_discovery_complete(struct audio_device *dev, void *user_data
 	pcm->sampling_rate = 8000;
 	if (headset_get_nrec(dev))
 		pcm->flags |= BT_PCM_FLAG_NREC;
+	if (!headset_get_sco_hci(dev))
+		pcm->flags |= BT_PCM_FLAG_PCM_ROUTING;
 
 	rsp->h.type = BT_RESPONSE;
 	rsp->h.name = BT_GET_CAPABILITIES;
@@ -309,7 +315,9 @@ static void headset_resume_complete(struct audio_device *dev, void *user_data)
 	if (!dev)
 		goto failed;
 
-	if (!headset_lock(dev, hs->lock)) {
+	if (!hs->locked)
+		hs->locked = headset_lock(dev, hs->lock);
+	if (!hs->locked) {
 		error("Unable to lock headset");
 		goto failed;
 	}
@@ -318,6 +326,7 @@ static void headset_resume_complete(struct audio_device *dev, void *user_data)
 	if (client->data_fd < 0) {
 		error("Unable to get a SCO fd");
 		headset_unlock(dev, hs->lock);
+		hs->locked = FALSE;
 		goto failed;
 	}
 
@@ -338,6 +347,7 @@ static void headset_resume_complete(struct audio_device *dev, void *user_data)
 	if (unix_sendmsg_fd(client->sock, client->data_fd) < 0) {
 		error("unix_sendmsg_fd: %s(%d)", strerror(errno), errno);
 		headset_unlock(client->dev, hs->lock);
+		hs->locked = FALSE;
 		goto failed;
 	}
 
@@ -352,7 +362,7 @@ static void headset_suspend_complete(struct audio_device *dev, void *user_data)
 {
 	struct unix_client *client = user_data;
 	char buf[BT_SUGGESTED_BUFFER_SIZE];
-	struct bt_start_stream_rsp *rsp = (void *) buf;
+	struct bt_stop_stream_rsp *rsp = (void *) buf;
 
 	if (!dev)
 		goto failed;
@@ -377,17 +387,19 @@ static int a2dp_append_codec(struct bt_get_capabilities_rsp *rsp,
 {
 	struct avdtp_media_codec_capability *codec_cap = (void *) cap->data;
 	codec_capabilities_t *codec = (void *) rsp + rsp->h.length;
-	int space_left = BT_SUGGESTED_BUFFER_SIZE - rsp->h.length;
+	size_t space_left;
 
-	if (space_left <= 0)
+	if (rsp->h.length > BT_SUGGESTED_BUFFER_SIZE)
 		return -ENOMEM;
 
+	space_left = BT_SUGGESTED_BUFFER_SIZE - rsp->h.length;
+
 	/* endianess prevent direct cast */
 	if (codec_cap->media_codec_type == A2DP_CODEC_SBC) {
 		struct sbc_codec_cap *sbc_cap = (void *) codec_cap;
 		sbc_capabilities_t *sbc = (void *) codec;
 
-		if ((space_left - (int) sizeof(sbc_capabilities_t)) < 0)
+		if (space_left < sizeof(sbc_capabilities_t))
 			return -ENOMEM;
 
 		codec->length = sizeof(sbc_capabilities_t);
@@ -403,7 +415,7 @@ static int a2dp_append_codec(struct bt_get_capabilities_rsp *rsp,
 		struct mpeg_codec_cap *mpeg_cap = (void *) codec_cap;
 		mpeg_capabilities_t *mpeg = (void *) codec;
 
-		if ((space_left - (int) sizeof(mpeg_capabilities_t)) < 0)
+		if (space_left < sizeof(mpeg_capabilities_t))
 			return -ENOMEM;
 
 		codec->length = sizeof(mpeg_capabilities_t);
@@ -415,12 +427,12 @@ static int a2dp_append_codec(struct bt_get_capabilities_rsp *rsp,
 		mpeg->mpf = mpeg_cap->mpf;
 		mpeg->bitrate = mpeg_cap->bitrate;
 	} else {
-		int codec_length;
+		size_t codec_length;
 
 		codec_length = cap->length - (sizeof(struct avdtp_service_capability)
 				+ sizeof(struct avdtp_media_codec_capability));
 
-		if ((space_left - codec_length - (int) sizeof(codec_capabilities_t)) < 0)
+		if (space_left < codec_length + sizeof(codec_capabilities_t))
 			return -ENOMEM;
 
 		codec->length = codec_length + sizeof(codec_capabilities_t);
@@ -499,11 +511,12 @@ static void a2dp_config_complete(struct avdtp *session, struct a2dp_sep *sep,
 	uint16_t imtu, omtu;
 	GSList *caps;
 
+	client->req_id = 0;
+
 	if (err)
 		goto failed;
 
 	memset(buf, 0, sizeof(buf));
-	client->req_id = 0;
 
 	if (!stream)
 		goto failed;
@@ -513,6 +526,10 @@ static void a2dp_config_complete(struct avdtp *session, struct a2dp_sep *sep,
 		goto failed;
 	}
 
+	if (client->cb_id > 0)
+		avdtp_stream_remove_cb(a2dp->session, a2dp->stream,
+								client->cb_id);
+
 	a2dp->sep = sep;
 	a2dp->stream = stream;
 
@@ -613,7 +630,7 @@ static void a2dp_suspend_complete(struct avdtp *session,
 {
 	struct unix_client *client = user_data;
 	char buf[BT_SUGGESTED_BUFFER_SIZE];
-	struct bt_start_stream_rsp *rsp = (void *) buf;
+	struct bt_stop_stream_rsp *rsp = (void *) buf;
 	struct a2dp_data *a2dp = &client->d.a2dp;
 
 	if (err)
@@ -795,6 +812,9 @@ static void start_resume(struct audio_device *dev, struct unix_client *client)
 		goto failed;
 	}
 
+	client->req_id = id;
+	client->dev = dev;
+
 	return;
 
 failed:
@@ -1099,8 +1119,10 @@ static gboolean client_cb(GIOChannel *chan, GIOCondition cond, gpointer data)
 		debug("Unix client disconnected (fd=%d)", client->sock);
 		switch (client->type) {
 		case TYPE_HEADSET:
-			if (client->dev)
+			if (client->dev && hs->locked) {
 				headset_unlock(client->dev, hs->lock);
+				hs->locked = FALSE;
+			}
 			break;
 		case TYPE_SOURCE:
 		case TYPE_SINK:
@@ -1255,6 +1277,8 @@ void unix_exit(void)
 {
 	g_slist_foreach(clients, (GFunc) client_free, NULL);
 	g_slist_free(clients);
-	close(unix_sock);
-	unix_sock = -1;
+	if (unix_sock >= 0) {
+		close(unix_sock);
+		unix_sock = -1;
+	}
 }
